package edu.berkeley.cs160.happytoastmobile.BorgTV;


import java.util.HashMap;

import android.content.*;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteQueryBuilder;
import android.net.Uri;
import android.text.TextUtils;
import android.util.Log;

public class InfoProvider extends ContentProvider {

	private static final String TAG = "InfoProvider";
	public static final Uri CONTENT_URI = Uri.parse("content://edu.berkeley.edu.cs160.happytoastmobile.BorgTV.DataDefs");
	
	private static final int LOC = 1;
	private static final int ROOM = 2;
	private static final int DEVICE = 3;
	private static final int LOC_I = 4;
	private static final int ROOM_I = 5;
	private static final int DEVICE_I = 6;

	private static final String DATABASE_NAME = "borgtv.db";
	private static final String DB_TABLE_LOC = "locations";
	private static final String DB_TABLE_ROOM = "rooms";
	private static final String DB_TABLE_DEVICE = "devices";
	private static final int DATABASE_VERSION = 13;

	private SQLiteDatabase mDB;
	private static final UriMatcher URL_MATCHER;


	private static class DatabaseHelper extends SQLiteOpenHelper {
		@Override
		public void onCreate(SQLiteDatabase db) {
			db.execSQL("CREATE TABLE locations (_id INTEGER PRIMARY KEY AUTOINCREMENT,"
					+ "location TEXT NOT NULL, locPlacement INTEGER);");
			db.execSQL("CREATE TABLE rooms (_id INTEGER PRIMARY KEY AUTOINCREMENT, loc_id INTEGER,"
					+ "room TEXT NOT NULL, roomPlacement INTEGER);");
			db.execSQL("CREATE TABLE devices (_id INTEGER PRIMARY KEY AUTOINCREMENT, room_id INTEGER,"
			        + "device TEXT NOT NULL, devPlacement INTEGER);");
		}

		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
			Log.w(TAG, "Upgrading database from version " + oldVersion + " to "
					+ newVersion + ", which will destroy all old data");
			db.execSQL("DROP TABLE IF EXISTS " + DB_TABLE_LOC);
	        db.execSQL("DROP TABLE IF EXISTS " + DB_TABLE_ROOM);
	        db.execSQL("DROP TABLE IF EXISTS " + DB_TABLE_DEVICE);
			onCreate(db);
		}
	}
	@Override
	public boolean onCreate() {
		DatabaseHelper dbHelper = new DatabaseHelper();
		mDB = dbHelper.openDatabase(getContext(), DATABASE_NAME, null, DATABASE_VERSION);
		return (mDB == null) ? false : true;
	}

	@Override
	public Cursor query(Uri url, String[] projection, String selection,
			String[] selectionArgs, String sort) {
        String[] selArgs;
        
		switch (URL_MATCHER.match(url)) {
		case LOC:
		    return mDB.rawQuery("SELECT location, locPlacement FROM locations",null);

		case ROOM:
		    selArgs = new String[1];
		    selArgs[0] = url.getPathSegments().get(1);
		    return mDB.rawQuery("SELECT rooms.room, rooms.roomPlacement FROM locations, rooms WHERE locations.location=?"
                              + " AND locations._id=rooms.loc_id" ,selArgs);

		case DEVICE:
		    selArgs = new String[2];
            selArgs[0] = url.getPathSegments().get(1);
            selArgs[1] = url.getPathSegments().get(3);
            return mDB.rawQuery("SELECT devices.device, devices.devPlacement FROM locations, rooms, devices "
                              + "WHERE locations.location=? "
                              + "AND locations._id=rooms.loc_id "
                              + "AND rooms.room=?"
                              + "AND rooms._id=devices.room_id",selArgs);
		    
		default:
			throw new IllegalArgumentException("Unknown URL " + url);
		}
	}
	
    @Override
    public String getType(Uri url) {
        switch (URL_MATCHER.match(url)) {
        case LOC:
            return "vnd.android.cursor.dir/vnd.happytoast.location";

        case ROOM:
            return "vnd.android.cursor.dir/vnd.happytoast.room";
            
        case DEVICE:
            return "vnd.android.cursor.dir/vnd.happytoast.device";

        default:
            throw new IllegalArgumentException("Unknown URL " + url);
        }
    }
    
    @Override
    public Uri insert(Uri url, ContentValues initialValues) {
        long rowID;
        String[] selArgs;
        ContentValues values;
        if (initialValues != null) {
            values = new ContentValues(initialValues);
        } else {
            values = new ContentValues();
        }
        
        if (URL_MATCHER.match(url) == LOC_I) {
            values.put(DataDefs.Fields.LOCATION, url.getPathSegments().get(1));
            values.put("locPlacement", url.getPathSegments().get(2));
            rowID = mDB.insert(DB_TABLE_LOC, "location", values);
            
        } else if (URL_MATCHER.match(url) == ROOM_I){
            selArgs = new String[1];
            selArgs[0] = url.getPathSegments().get(1);
            Cursor c = mDB.rawQuery("SELECT _id FROM locations WHERE location=?", selArgs);
            
            if(!c.first())
                Log.e("FAIL!", "Insert Room cursor is EMPTY! Location doesn't exist?");

            values.put("loc_id",c.getString(c.getColumnIndex("_id")));
            values.put(DataDefs.Fields.ROOM, url.getPathSegments().get(3));
            values.put("roomPlacement", url.getPathSegments().get(4));

            rowID = mDB.insert(DB_TABLE_ROOM, "room", values);
            
            c.close();
            
        } else if (URL_MATCHER.match(url) == DEVICE_I){
            selArgs = new String[2];
            selArgs[0] = url.getPathSegments().get(1);
            selArgs[1] = url.getPathSegments().get(3);
            Cursor c = mDB.rawQuery("SELECT rooms._id FROM locations, rooms "
                                  + "WHERE location=? AND locations._id=rooms.loc_id "
                                  + "AND rooms.room=?", selArgs);
            
            if(!c.first())
                Log.e("FAIL!", "Insert Device cursor is EMPTY! Room doesn't exist?");
            
            values.put("room_id", c.getString(0));
            values.put(DataDefs.Fields.DEVICE, url.getPathSegments().get(5));
            values.put("devPlacement", url.getPathSegments().get(6));
            
            rowID = mDB.insert(DB_TABLE_DEVICE, "device", values);
            c.close();
        } else {
            throw new IllegalArgumentException("Unknown URL for insert: " + url);
        }

        if (rowID > 0) {
            Uri uri = ContentUris.withAppendedId(DataDefs.Fields.CONTENT_URI, rowID);
            getContext().getContentResolver().notifyChange(uri, null);
            return uri;
        }

        throw new SQLException("Failed to insert row into " + url);
    }

    
    @Override
    public int delete(Uri url, String where, String[] whereArgs) {
        int count;
        String loc, room, dev,loc_id,room_id;
        String[] selArgs;
        Cursor  c;
        switch (URL_MATCHER.match(url)) {
        case LOC_I:
            loc = url.getPathSegments().get(1);
            selArgs = new String[1];
            selArgs[0] = loc;
            c = mDB.rawQuery("SELECT _id FROM locations WHERE location=?",selArgs);
            if(!c.first())
                Log.e("FAIL!", "Delete loc cursor is EMPTY! Location doesn't exist?");
            loc_id=c.getString(0);
            c.close();
            
            selArgs[0] = loc_id;
            c = mDB.rawQuery("SELECT _id FROM rooms WHERE loc_id=?", selArgs);
            count = 0;
            if(!c.first())
                Log.e("FAIL!", "Delete loc cursor is EMPTY! Rooms don't exist?");
            for(int x = 0; x < c.count(); x++){
                room_id=c.getString(0);
                selArgs[0] = room_id;
                count += mDB.delete("devices", "room_id=?", selArgs);
                if(!c.next()){
                    break;
                }
            }
            c.close();
            selArgs[0] = loc_id;
            count += mDB.delete("rooms", "loc_id=?", selArgs);
            
            //Delete lsc
            selArgs[0]=loc;
            count += mDB.delete("locations", "location=?", selArgs);
            break;
           
        case ROOM_I:
            loc = url.getPathSegments().get(1);
            room = url.getPathSegments().get(3);
            selArgs = new String[1];
            selArgs[0] = loc;
            c = mDB.rawQuery("SELECT _id FROM locations WHERE location=?",selArgs);
            if(!c.first())
                Log.e("FAIL!", "Delete Room cursor is EMPTY! Location doesn't exist?");
            loc_id=c.getString(0);
            c.close();
            
            selArgs = new String[2];
            selArgs[0] = room;
            selArgs[1] = loc_id;
            c = mDB.rawQuery("SELECT _id FROM rooms WHERE room=? AND loc_id=?", selArgs);
            if(!c.first())
                Log.e("FAIL!", "Delete Room cursor is EMPTY! Room doesn't exist?");
            room_id=c.getString(0);
            c.close();
            //Delete the devices associated with this room
            selArgs[0] = room_id;
            count = mDB.delete("devices", "room_id=?", selArgs);
            
            //Delete this room
            selArgs[0] = loc_id;
            selArgs[1] = room;
            count += mDB.delete("rooms", "loc_id=? AND room=?", selArgs);
            break;
            
        case DEVICE_I:
            loc = url.getPathSegments().get(1);
            room = url.getPathSegments().get(3);
            dev = url.getPathSegments().get(5);
            
            selArgs = new String[1];
            selArgs[0] = loc;
            c = mDB.rawQuery("SELECT _id FROM locations WHERE location=?",selArgs);
            if(!c.first())
                Log.e("FAIL!", "Delete Room cursor is EMPTY! Location doesn't exist?");
            loc_id=c.getString(0);
            c.close();
            
            selArgs = new String[2];
            selArgs[0] = room;
            selArgs[1] = loc_id;
            c = mDB.rawQuery("SELECT _id FROM rooms WHERE room=? AND loc_id=?", selArgs);
            if(!c.first())
                Log.e("FAIL!", "Delete Room cursor is EMPTY! Room doesn't exist?");
            room_id=c.getString(0);
            c.close();
            
            selArgs[0] = room_id;
            selArgs[1] = dev;
            count = mDB.delete("devices", "room_id=? AND device=?", selArgs);
            break;

        default:
            throw new IllegalArgumentException("Unknown URL " + url);
        }

        getContext().getContentResolver().notifyChange(url, null);
        return count;
    }

    @Override
    public int update(Uri url, ContentValues values, String where, String[] whereArgs) {
        int count;
        Cursor c;
        String loc, room, dev, loc_id, room_id;
        String[] selArgs;
        switch (URL_MATCHER.match(url)) {
        case LOC_I:
            loc = url.getPathSegments().get(1);
            selArgs = new String[1];
            selArgs[0] = loc;    
            count = mDB.update("locations", values, "location=?", selArgs);
            break;
           
        case ROOM_I:
            selArgs = new String[1];
            loc = url.getPathSegments().get(1);
            room = url.getPathSegments().get(3);
            
            
            selArgs[0] = loc;
            c = mDB.rawQuery("SELECT _id FROM locations WHERE location=?",selArgs);
            if(!c.first())
                Log.e("FAIL!", "Delete Room cursor is EMPTY! Location doesn't exist?");
            loc_id=c.getString(0);
            c.close();

            selArgs = new String[2];
            selArgs[0] = loc_id;
            selArgs[1] = room;
            count = mDB.update("rooms", values, "loc_id=? AND room=?", selArgs);
            break;
        
        case DEVICE_I:
            selArgs = new String[2];
            loc = url.getPathSegments().get(1);
            room = url.getPathSegments().get(3);
            dev = url.getPathSegments().get(5);
            
            selArgs[0] = loc;
            c = mDB.rawQuery("SELECT _id FROM locations WHERE location=?",selArgs);
            if(!c.first())
                Log.e("FAIL!", "Delete Room cursor is EMPTY! Location doesn't exist?");
            loc_id=c.getString(0);
            c.close();
            
            selArgs[0] = loc_id;
            selArgs[1] = room;
            c = mDB.rawQuery("SELECT _id FROM rooms WHERE room=? AND loc_id=?", selArgs);
            if(!c.first())
                Log.e("FAIL!", "Delete Room cursor is EMPTY! Room doesn't exist?");
            room_id=c.getString(0);
            c.close();
            
            selArgs[0]=room_id;
            selArgs[1]=dev;
            count = mDB.update("devices", values, "room_id=? AND device=?", selArgs);
            break;
            
        default:
            throw new IllegalArgumentException("Unknown URL " + url);
        }

        getContext().getContentResolver().notifyChange(url, null);
        return count;
    }

	static {
		URL_MATCHER = new UriMatcher(UriMatcher.NO_MATCH);
		URL_MATCHER.addURI("edu.berkeley.cs160.happytoastmobile.BorgTV.DataDefs", "locations", LOC);
		URL_MATCHER.addURI("edu.berkeley.cs160.happytoastmobile.BorgTV.DataDefs", "locations/*/#", LOC_I);
		// Matches a request for the rooms given a location. CAN BE NULL (how will this work) 
        URL_MATCHER.addURI("edu.berkeley.cs160.happytoastmobile.BorgTV.DataDefs", "locations/*/rooms", ROOM);
        URL_MATCHER.addURI("edu.berkeley.cs160.happytoastmobile.BorgTV.DataDefs", "locations/*/rooms/*/#", ROOM_I);

        // Matches a request for the devices given a location and room. If the loc has no rooms, the room field will be empty
        URL_MATCHER.addURI("edu.berkeley.cs160.happytoastmobile.BorgTV.DataDefs", "locations/*/rooms/*/devices/", DEVICE);
        URL_MATCHER.addURI("edu.berkeley.cs160.happytoastmobile.BorgTV.DataDefs", "locations/*/rooms/*/devices/*/#", DEVICE_I);

	}

}
