package edu.berkeley.cs160.happytoastmobile.BorgTV.NineButton;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.ContentValues;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;
import edu.berkeley.cs160.happytoastmobile.BorgTV.DataDefs;
import edu.berkeley.cs160.happytoastmobile.BorgTV.R;
import edu.berkeley.cs160.happytoastmobile.BorgTV.Setup.Setup;

public class MenuBorgTV extends Activity {
    public final static String newLocation = "New Location...";
	Cursor data;
	private int currSetupPlacement, currSetButtons;
	private String editAble;
	private int editablePlacement;
	private SetNineButtons basicButtons;
	
	//All the locations for more menu
	private Bundle myLocations;

	public void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		setContentView(R.layout.ninebuttonlayout);
		this.setTitle("BorgTV");
		
		myLocations = new Bundle();
		
		basicButtons = new SetNineButtons(this);
		basicButtons.back.requestFocus();
		basicButtons.center.setText(newLocation);
		basicButtons.up.setText(newLocation);
		basicButtons.down.setText(newLocation);
		basicButtons.left.setText(newLocation);
		basicButtons.right.setText(newLocation);		
		basicButtons.help.setOnClickListener(new Button.OnClickListener() {
			public void onClick(View v) {
				help();
			}
		});

		basicButtons.more.setOnClickListener(new Button.OnClickListener() {
			public void onClick(View v) {
				more();
			}
		});

		Uri uri = DataDefs.Fields.mkQueryUri();
		data = this.managedQuery(uri, null, null, null);
		propagate();
		setButtons();
		
		if (data.count() == 1 && data.first()) {
			goIntoNextLevel(data.getString(0), true);
		} else if (!data.first()){
		    Toast welcome = Toast.makeText(this, R.string.welcome_toast, Toast.LENGTH_SHORT);
		    welcome.show();
		    Intent i = new Intent(this, Setup.class);
		    i.putExtra("placement", SetNineButtons.CENTER);
		    i.setAction(Setup.ADD_LOC_ACTION_FIRST);
		    startSubActivity(i,0); 
		    currSetupPlacement = SetNineButtons.CENTER;
		}
		
		moreCheckClickable();
		
	}
	
	void onResume(Bundle icicle) {
		super.onResume();
        moreCheckClickable();
		propagate();
	}
	
	boolean moreCheckClickable() {
	    data.requery();
	    if(data.count() < 5) {
            basicButtons.more.setTextColor(Color.GRAY);
            basicButtons.more.setFocusable(false);
            return false;
        } else {
            basicButtons.more.setTextColor(Color.BLACK);
            basicButtons.more.setFocusable(true);
            return true;
        }
	}

	void setButtons() {
	    for (int i = 0; i < 5; i ++) {
	        if(basicButtons.getButton(i).getText().equals(newLocation))
	            basicButtons.getButton(i).setOnClickListener(new Button.OnClickListener() {
	                public void onClick(View v) {
	                    setup(basicButtons.getCode(v.getId()));
	                }
	            });
	        else 
	            basicButtons.getButton(i).setOnClickListener(new Button.OnClickListener() {
	                public void onClick(View v) {
	                    goIntoNextLevel (basicButtons.getButton(v.getId()).getText(), false);
	                }
	            });
	    }
	}
	
	void propagate() {
		if(data.first())
			for(int i = 0; i < data.count(); i++) {
				String nameRoom = data.getString(0);
				int placement = data.getInt(1);
				
				if (!nameRoom.equals(newLocation))
					myLocations.putString(nameRoom, nameRoom);

				switch(placement) {
				case SetNineButtons.CENTER:
					basicButtons.center.setText(nameRoom);
					break;
				case SetNineButtons.UP:
					basicButtons.up.setText(nameRoom);
					break;
				case SetNineButtons.DOWN:
					basicButtons.down.setText(nameRoom);
					break;
				case SetNineButtons.LEFT:
					basicButtons.left.setText(nameRoom);
					break;
				case SetNineButtons.RIGHT:
					basicButtons.right.setText(nameRoom);
					break;
				default:
					break;
				}
				if (!data.next())
					break;
			}
	}

	void setup (int placement) {
		Intent i = new Intent(this, Setup.class);
		i.putExtra("placement", placement);
		i.setAction(Setup.ADD_LOC_ACTION);
		startSubActivity(i, 0);	
		currSetupPlacement = placement;

	}

	void help () {
		Intent help = new Intent(this, Help.class);
		help.putExtra("caller", "BorgTV");
		help.putExtra("message", "put some stuff here");
		startSubActivity(help, 0);
	}

	void more() {
		Intent i = new Intent(this, More.class);
		i.putExtra("caller", "BorgTV");
		i.putExtra("message", "put some stuff here");
		i.putExtra("list", myLocations);
		i.putExtra("contains", "Location");
		i.putExtra("resultList", createActivityPendingResult(Codes.LOCATION, true));
		startSubActivity(i, Codes.LOCATION);
		
	}
 
	void goIntoNextLevel (CharSequence loc, boolean first) {
		Intent nextLevel = new Intent(this, MenuLocation.class);
		nextLevel.putExtra("loc", loc);
		
		if(first) {
			nextLevel.putExtra("first", true);
		}
		
		startSubActivity(nextLevel, 0);
	}

	protected void onActivityResult(int requestCode, int resultCode,
			String data, Bundle extras) {
	    super.onActivityResult(requestCode, resultCode, data, extras);
	    moreCheckClickable();
	    
		if(resultCode == RESULT_OK) {
		    if (currSetupPlacement != 6)
		        basicButtons.getButton(currSetupPlacement).setText(data);
		    myLocations.putString(data, data);
			goIntoNextLevel(data, false);
			setButtons();
		} else if (resultCode == Codes.SELECT) {
			goIntoNextLevel(data, false);
		} else if (resultCode == Codes.DELETE) {
		   delete(data);
		} else if (resultCode == Codes.NEW) {
		    setup(6);
		    currSetupPlacement = 6;
		} else if (resultCode == Codes.RENAME_OK) {	    
		    if (data != null) {
		       rename(data, extras.getInt("placement"));
		    }
		} else if (resultCode == Codes.RENAME_MORE) {
		    renameMore (extras.getString("old"),
		                extras.getString("new"));
		}
	}
	
	public boolean onKeyUp(int keyCode, KeyEvent event) {
        switch (keyCode) {
            case KeyEvent.KEYCODE_1:
            case KeyEvent.KEYCODE_W:
                basicButtons.guide();
                break;
            case KeyEvent.KEYCODE_2:
            case KeyEvent.KEYCODE_E:
                if(basicButtons.up.getText().equals(newLocation)) 
                    setup(SetNineButtons.UP);
                else
                    goIntoNextLevel(basicButtons.up.getText(), false);
                break;
            case KeyEvent.KEYCODE_3:
            case KeyEvent.KEYCODE_R:
                help();
                break;
            case KeyEvent.KEYCODE_4:
            case KeyEvent.KEYCODE_S:
                if(basicButtons.center.getText().equals(newLocation)) 
                    setup(SetNineButtons.LEFT);
                else
                    goIntoNextLevel(basicButtons.left.getText(), false);
                break;
            case KeyEvent.KEYCODE_5:
            case KeyEvent.KEYCODE_D:
                if(basicButtons.center.getText().equals(newLocation)) 
                    setup(SetNineButtons.CENTER);
                else
                    goIntoNextLevel(basicButtons.center.getText(), false);
                break;
                
            case KeyEvent.KEYCODE_6:
            case KeyEvent.KEYCODE_F:               
                if(basicButtons.right.getText().equals(newLocation)) 
                    setup(SetNineButtons.RIGHT);
                else
                    goIntoNextLevel(basicButtons.right.getText(), false);
                break;
                
            case KeyEvent.KEYCODE_7:
            case KeyEvent.KEYCODE_Z:
                this.finish();
                break;
                
            case KeyEvent.KEYCODE_8:
            case KeyEvent.KEYCODE_X:
                if(basicButtons.down.getText().equals(newLocation)) 
                    setup(SetNineButtons.DOWN);
                else
                    goIntoNextLevel(basicButtons.down.getText(), false);
                break;              
                
            case KeyEvent.KEYCODE_9:
            case KeyEvent.KEYCODE_C:
                if (moreCheckClickable())
                    more();
                break;
            }
        return super.onKeyUp(keyCode, event);
    }
	

    public boolean onPrepareOptionsMenu(Menu menu) {
        super.onPrepareOptionsMenu(menu);
        boolean failed = false;
        for (int i = 0; i < 5; i++) {
            if (basicButtons.getButton(i).hasFocus()) {
                editAble = (String) basicButtons.getButton(i).getText();
                editablePlacement = i;
                break;
            }
            if (i == 4) {
                failed = true;
            }
        }
        menu.clear();
        if(!failed && !(editAble.equals(newLocation))) {
            menu.add(0, 0, "Rename");
            menu.add(0, 1, "Delete");
            return true;
        } else
            return false;
    }
	
	public boolean onOptionsItemSelected(Menu.Item item) {

	    switch (item.getId()) {
	    case 0:
	        Intent i = new Intent(this, Edit.class);
	        i.putExtra("placement", editablePlacement);
            i.putExtra("caller", "Location");
	        startSubActivity(i, 0);
	        return true;
	    case 1:
	        AlertDialog.show(this, "Delete",
                    R.drawable.alert, "Do you really want to delete "+editAble+"?", 
                    "Yes", new DialogInterface.OnClickListener(){public void onClick(DialogInterface d,int i){
                        delete(editAble);
                    }},
                    "No", new DialogInterface.OnClickListener(){public void onClick(DialogInterface d,int i){
                    }},
                    false, null);
	        return true;
	    }
	    return false;
	}
	
	void delete (String name) {
        getContentResolver().delete(DataDefs.Fields.mkInsertUri(name, 0), null, null);
        Toast toasty = Toast.makeText(this, name+" has been deleted.", Toast.LENGTH_SHORT);
        toasty.show();
        onCreate(null);
	}
	
	void rename (String name, int placement) {        
        //Change name within database
        ContentValues newName = new ContentValues();
        newName.put("location", name);
        CharSequence oldName = basicButtons.getButton(placement).getText();

        Uri oldValue = DataDefs.Fields.mkInsertUri(oldName.toString(), 0);
        getContentResolver().update(oldValue, newName, null, null);
        basicButtons.getButton(placement).setText(name);
        onCreate(null);
	}
	
	void renameMore (String oldName, String newName) {
        Uri oldValue = DataDefs.Fields.mkInsertUri(oldName, 0);
        ContentValues newValues = new ContentValues();
        newValues.put("location", newName);
        getContentResolver().update(oldValue, newValues, null, null);        
        
        for (int i = 0; i < 5; i++) {
            if (basicButtons.getButton(i).getText().equals(oldName)) {
                basicButtons.getButton(i).setText(newName);
                break;
            }
        }
        onCreate(null);
	}
}
