package edu.berkeley.cs160.happytoastmobile.BorgTV.NineButton;

import java.util.HashMap;

import edu.berkeley.cs160.happytoastmobile.BorgTV.DataDefs;
import edu.berkeley.cs160.happytoastmobile.BorgTV.R;
import edu.berkeley.cs160.happytoastmobile.BorgTV.Setup.Setup;

import android.app.Activity;
import android.app.AlertDialog;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;
import android.content.ContentValues;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Color;

public class MenuRoom extends Activity {
    Cursor data;
    private int currSetupPlacement, editablePlacement;
    private SetNineButtons basicButtons;
    private Bundle myDevices = new Bundle();
    String loc, room;
    private String editAble;
    public static final String newDevice = "New Device...";

    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        setContentView(R.layout.ninebuttonlayout);
        this.findViewById(R.id.backgroundLayout).setBackground(
                R.drawable.brownback);

        basicButtons = new SetNineButtons(this);

        basicButtons.center.setText(newDevice);
        basicButtons.up.setText(newDevice);
        basicButtons.down.setText(newDevice);
        basicButtons.left.setText(newDevice);
        basicButtons.right.setText(newDevice);

        basicButtons.help.setOnClickListener(new Button.OnClickListener() {
            public void onClick(View v) {
                help();
            }
        });

        basicButtons.more.setOnClickListener(new Button.OnClickListener() {
            public void onClick(View v) {
                more();
            }
        });

        basicButtons.back.requestFocus();

        Intent i = getIntent();
        room = i.getStringExtra("room");
        loc = i.getStringExtra("loc");

        Uri uri = DataDefs.Fields.mkQueryUri(loc, room);
        data = this.managedQuery(uri, null, null, null);
        propagate();
        setButtons();
        checkTitle();
        String bonusDevice = i.getStringExtra("bonusDevice");

        if (bonusDevice != null) {
            setup(SetNineButtons.CENTER, bonusDevice);
        }
        if (data.count() == 1 && data.first()) {
            goIntoNextLevel(data.getString(0));
        }

        moreCheckClickable();
    }

    boolean moreCheckClickable() {
        data.requery();
        if (data.count() < 5) {
            basicButtons.more.setTextColor(Color.GRAY);
            basicButtons.more.setFocusable(false);
            return false;
        } else {
            basicButtons.more.setTextColor(Color.BLACK);
            basicButtons.more.setFocusable(true);
            return true;
        }
    }

    boolean checkOneRoom() {
        Intent i = getIntent();
        return i.getBooleanExtra("oneRoom", false);
    }

    void checkTitle() {
        if (checkOneRoom()) {
            this.setTitle("BorgTV > " + loc);
        } else {
            this.setTitle("BorgTV > " + loc + " > " + room);
        }
    }

    void onResume(Bundle icicle) {
        super.onResume();
        setButtons();
        if (data.count() < 5) {
            basicButtons.more.setTextColor(Color.GRAY);
            basicButtons.more.setFocusable(false);
        } else {
            basicButtons.more.setTextColor(Color.BLACK);
            basicButtons.more.setFocusable(true);
        }
    }

    void setButtons() {
        for (int i = 0; i < 5; i++) {
            if (basicButtons.getButton(i).getText().equals(newDevice))
                basicButtons.getButton(i).setOnClickListener(
                        new Button.OnClickListener() {
                            public void onClick(View v) {
                                setup(basicButtons.getCode(v.getId()));
                            }
                        });
            else
                basicButtons.getButton(i).setOnClickListener(
                        new Button.OnClickListener() {
                            public void onClick(View v) {
                                goIntoNextLevel(basicButtons.getButton(
                                        v.getId()).getText());
                            }
                        });
        }
    }

    void propagate() {
        if (data.first())
            for (int i = 0; i < data.count(); i++) {
                String nameRoom = data.getString(0);
                int placement = data.getInt(1);

                if (!nameRoom.equals(newDevice))
                    myDevices.putString(Integer.toString(placement), nameRoom);

                switch (placement) {
                case SetNineButtons.CENTER:
                    basicButtons.center.setText(nameRoom);
                    break;
                case SetNineButtons.UP:
                    basicButtons.up.setText(nameRoom);
                    break;
                case SetNineButtons.DOWN:
                    basicButtons.down.setText(nameRoom);
                    break;
                case SetNineButtons.LEFT:
                    basicButtons.left.setText(nameRoom);
                    break;
                case SetNineButtons.RIGHT:
                    basicButtons.right.setText(nameRoom);
                    break;
                default:
                    break;
                }
                if (!data.next())
                    break;
            }
    }

    void setup(int placement) {
        Intent i = new Intent(this, Setup.class);
        i.setAction(Setup.ADD_DEVICE_ACTION);
        i.putExtra("placement", placement);
        Uri uri = DataDefs.Fields.mkQueryUri(loc, room);
        i.setData(uri);
        i.putExtra("oneRoom", checkOneRoom());
        startSubActivity(i, 0);
        currSetupPlacement = placement;

    }

    void setup(int placement, String bonusDevice) {
        Intent i = new Intent(this, Setup.class);
        i.setAction(Setup.ADD_DEVICE_ACTION);
        i.putExtra("placement", placement);
        i.putExtra("bonusDevice", bonusDevice);
        Uri uri = DataDefs.Fields.mkQueryUri(loc, room);
        i.setData(uri);
        startSubActivity(i, 0);
        currSetupPlacement = placement;
    }

    void help() {
        Intent help = new Intent(this, Help.class);
        help.putExtra("caller", "Rooms");
        help.putExtra("message", "how to setup Devices");
        startSubActivity(help, 0);
    }

    void more() {
        Intent i = new Intent(this, More.class);
        i.putExtra("caller", "Room");
        i.putExtra("message", "put some stuff here");
        i.putExtra("list", myDevices);
        i.putExtra("contains", "Device");
        i.putExtra("resultList", createActivityPendingResult(Codes.LOCATION,
                true));

        startSubActivity(i, 0);
    }

    void goIntoNextLevel(CharSequence device) {
        Intent nextLevel = new Intent(this, MenuDevices.class);
        nextLevel.putExtra("loc", loc);
        nextLevel.putExtra("room", room);
        nextLevel.putExtra("device", device);
        if (checkOneRoom()) {
            nextLevel.putExtra("oneRoom", true);
        }
        startSubActivity(nextLevel, 0);
    }

    protected void onActivityResult(int requestCode, int resultCode,
            String data, Bundle extras) {
        moreCheckClickable();
        checkTitle();
        if (resultCode == RESULT_OK) {
            if (currSetupPlacement != 6)
                basicButtons.getButton(currSetupPlacement).setText(data);
            myDevices.putString(data, data);
            goIntoNextLevel(data);
        } else if (resultCode == Codes.SELECT) {
            goIntoNextLevel(data);
        } else if (resultCode == Codes.SELECT) {
            goIntoNextLevel(data);
        } else if (resultCode == Codes.DELETE) {
            delete(data);
        } else if (resultCode == Codes.RENAME_OK) {
            if (data != null) {
                rename(data, extras.getInt("placement"));
            }
        } else if (resultCode == Codes.RENAME_MORE) {
            renameMore(extras.getString("old"), extras.getString("new"));
        }

        if (resultCode == Setup.DEVICE_COLLISION) {
            setResult(Setup.DEVICE_COLLISION, data);
            finish();
        }
        setButtons();
    }

    public boolean onKeyUp(int keyCode, KeyEvent event) {
        switch (keyCode) {
        case KeyEvent.KEYCODE_1:
        case KeyEvent.KEYCODE_W:
            basicButtons.guide();
            break;
        case KeyEvent.KEYCODE_2:
        case KeyEvent.KEYCODE_E:
            if (basicButtons.up.getText().equals(newDevice))
                setup(SetNineButtons.UP);
            else
                goIntoNextLevel(basicButtons.up.getText());
            break;
        case KeyEvent.KEYCODE_3:
        case KeyEvent.KEYCODE_R:
            help();
            break;
        case KeyEvent.KEYCODE_4:
        case KeyEvent.KEYCODE_S:
            if (basicButtons.center.getText().equals(newDevice))
                setup(SetNineButtons.LEFT);
            else
                goIntoNextLevel(basicButtons.left.getText());
            break;
        case KeyEvent.KEYCODE_5:
        case KeyEvent.KEYCODE_D:
            if (basicButtons.center.getText().equals(newDevice))
                setup(SetNineButtons.CENTER);
            else
                goIntoNextLevel(basicButtons.center.getText());
            break;

        case KeyEvent.KEYCODE_6:
        case KeyEvent.KEYCODE_F:
            if (basicButtons.right.getText().equals(newDevice))
                setup(SetNineButtons.RIGHT);
            else
                goIntoNextLevel(basicButtons.right.getText());
            break;

        case KeyEvent.KEYCODE_7:
        case KeyEvent.KEYCODE_Z:
            this.finish();
            break;

        case KeyEvent.KEYCODE_8:
        case KeyEvent.KEYCODE_X:
            if (basicButtons.down.getText().equals(newDevice))
                setup(SetNineButtons.DOWN);
            else
                goIntoNextLevel(basicButtons.down.getText());
            break;

        case KeyEvent.KEYCODE_9:
        case KeyEvent.KEYCODE_C:
            if (moreCheckClickable())
                more();
            break;
        }
        return super.onKeyUp(keyCode, event);
    }

    public boolean onPrepareOptionsMenu(Menu menu) {
        super.onPrepareOptionsMenu(menu);
        boolean failed = false;
        for (int i = 0; i < 5; i++) {
            if (basicButtons.getButton(i).hasFocus()) {
                editAble = (String) basicButtons.getButton(i).getText();
                editablePlacement = i;
                break;
            }
            if (i == 4) {
                failed = true;
            }
        }
        menu.clear();
        if (!failed && !(editAble.equals(newDevice))) {
            menu.add(0, 1, "Delete");
            return true;
        } else
            return false;
    }

    public boolean onOptionsItemSelected(Menu.Item item) {

        switch (item.getId()) {
        case 0:
            Intent i = new Intent(this, Edit.class);
            i.putExtra("placement", editablePlacement);
            i.putExtra("caller", "Device");
            startSubActivity(i, 0);
            return true;
        case 1:
            AlertDialog.show(this, "Delete", R.drawable.alert,
                    "Do you really want to delete " + editAble + "?", "Yes",
                    new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface d, int i) {
                            delete(editAble);
                        }
                    }, "No", new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface d, int i) {
                        }
                    }, false, null);
            return true;
        }
        return false;
    }

    void delete(String name) {
        getContentResolver().delete(
                DataDefs.Fields.mkInsertUri(loc, room, name, 0), null, null);
        Toast toasty = Toast.makeText(this, name + " has been deleted.",
                Toast.LENGTH_SHORT);
        toasty.show();
        onCreate(null);
    }

    void rename(String name, int placement) {
        // Change name within database
        ContentValues newName = new ContentValues();
        newName.put("device", name);
        CharSequence oldName = basicButtons.getButton(placement).getText();

        Uri oldValue = DataDefs.Fields.mkInsertUri(loc, room, oldName
                .toString(), 0);
        getContentResolver().update(oldValue, newName, null, null);
        basicButtons.getButton(placement).setText(name);
        onCreate(null);
    }
    
    void renameMore (String oldName, String newName) {
        Uri oldValue = DataDefs.Fields.mkInsertUri(loc, room, oldName, 0);
        ContentValues newValues = new ContentValues();
        newValues.put("device", newName);
        getContentResolver().update(oldValue, newValues, null, null);        
        
        for (int i = 0; i < 5; i++) {
            if (basicButtons.getButton(i).getText().equals(oldName)) {
                basicButtons.getButton(i).setText(newName);
                break;
            }
        }
        onCreate(null);
    }
}