package edu.berkeley.cs160.happytoastmobile.BorgTV.NineButton;

import android.app.Activity;
import android.content.Intent;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import edu.berkeley.cs160.happytoastmobile.BorgTV.R;
import edu.berkeley.cs160.happytoastmobile.BorgTV.TVGuide.TVGuide;

/**
 * Defines the set four buttons (Guide, Back, Help and More) placed in the
 * corner of the NineButton menu.
 * 
 * @author Lita Cho
 * 
 */
public class SetNineButtons {
    // The activity that instantiated this object
    Activity thisActivity;

    static final int CENTER = 0;
    static final int LEFT = 1;
    static final int RIGHT = 2;
    static final int UP = 3;
    static final int DOWN = 4;

    // The four buttons that are in every NineButton menu
    Button guide, back, help, more;

    /*
     * Slots for the possible locations. All other locations not stored here
     * should be access through the More list.
     */
    Button center, up, down, left, right;

    /**
     * Constructs the four buttons that is set for the whole hierarchy. Each
     * NineButton menu must instantiate this class.
     * 
     * @param activity:
     *            Takes in the activity that is instantiating the class.
     */
    SetNineButtons(Activity activity) {
        thisActivity = activity;

        // Blank buttons
        center = (Button) thisActivity.findViewById(R.id.NineButtonCenter);
        up = (Button) thisActivity.findViewById(R.id.NineButtonUp);
        down = (Button) thisActivity.findViewById(R.id.NineButtonDown);
        left = (Button) thisActivity.findViewById(R.id.NineButtonLeft);
        right = (Button) thisActivity.findViewById(R.id.NineButtonRight);

        // Four corner buttons that are consistent
        guide = (Button) thisActivity.findViewById(R.id.NineButtonUpLeft);
        guide.setText("Guide");
        guide.setOnClickListener(new Button.OnClickListener() {
            public void onClick(View v) {
                guide();
            }
        });

        help = (Button) thisActivity.findViewById(R.id.NineButtonUpRight);
        help.setText("Help");

        back = (Button) thisActivity.findViewById(R.id.NineButtonDownLeft);
        back.setText("Back");
        back.setOnClickListener(new Button.OnClickListener() {
            public void onClick(View v) {
                thisActivity.finish();
            }
        });

        more = (Button) thisActivity.findViewById(R.id.NineButtonDownRight);
        more.setText("More");

    }

    Button getButton(int ButtonPlacement) {
        switch (ButtonPlacement) {
        case CENTER:
            return center;
        case R.id.NineButtonCenter:
            return center;
        case RIGHT:
        case R.id.NineButtonRight:
            return right;
        case LEFT:
        case R.id.NineButtonLeft:
            return left;
        case DOWN:
        case R.id.NineButtonDown:
            return down;
        case UP:
        case R.id.NineButtonUp:
            return up;
        }
        return null;
    }

    int getCode(int id) {
        switch (id) {
        case R.id.NineButtonCenter:
            return CENTER;
        case R.id.NineButtonRight:
            return RIGHT;
        case R.id.NineButtonLeft:
            return LEFT;
        case R.id.NineButtonDown:
            return DOWN;
        case R.id.NineButtonUp:
            return UP;
        default:
            return -1;
        }
    }

    void setText(String text, int buttonPlacement) {
        switch (buttonPlacement) {
        case CENTER:
            center.setText(text);
        case RIGHT:
            right.setText(text);
        case LEFT:
            left.setText(text);
        case DOWN:
            down.setText(text);
        case UP:
            up.setText(text);
        }
    }

    void guide() {
        Intent i = new Intent(thisActivity, TVGuide.class);
        thisActivity.startSubActivity(i, 0);
    }
}
