package edu.berkeley.cs160.happytoastmobile.BorgTV.Setup;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;
import edu.berkeley.cs160.happytoastmobile.BorgTV.DataDefs;
import edu.berkeley.cs160.happytoastmobile.BorgTV.R;
import edu.berkeley.cs160.happytoastmobile.BorgTV.NineButton.*;

public class Setup extends Activity {
    
	public static final int DEVICE_COLLISION = 616;
	public static final int BONUS_DEV = 617;
	
    public static final String ADD_LOC_ACTION = "add_loc_action";
    public static final String ADD_ROOM_ACTION = "add_room_action";
    public static final String ADD_DEVICE_ACTION = "add_dev_action";
    public static final String ADD_LOC_ACTION_FIRST = "add_loc_action_first";
    
    private String action;
    private String loc;
    private String room;
    private String device;
    private String brand;
    private String bonusDevice;
    private int deviceIndex;
    private int attempts;
    private int placement;
    private Toast emptyTextToast, maxAttemptsToast, irToast, successToast;
    private boolean inDevTestStart, inDevTestSend;
    
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        inDevTestStart = false;
        inDevTestSend = false;
        action = getIntent().getAction();
        bonusDevice = getIntent().getStringExtra("bonusDevice");

        if (action.equals(ADD_LOC_ACTION_FIRST)){
            setTheme(android.R.style.Theme);
            action = ADD_LOC_ACTION;
        }
        String editTextType = "";
        if(action.equals(ADD_LOC_ACTION)){
            setContentView(R.layout.setuploc);
            editTextType = "location";
        }else if (action.equals(ADD_ROOM_ACTION)){
            setContentView(R.layout.setuproom);
            editTextType = "room";
        } else if (action.equals(ADD_DEVICE_ACTION)){
            setContentView(R.layout.setupdevice);
            editTextType = "device brand"; 
        }else{
            Log.e("FAIL!","Setup called with invalid action");
            finish();
        }
        
        emptyTextToast = Toast.makeText(this, "Please enter a valid "+editTextType+" name", Toast.LENGTH_SHORT);
        maxAttemptsToast = Toast.makeText(this, "Sorry, that device is not currently supported by BorgTV", Toast.LENGTH_SHORT);
        Button createButton = (Button) findViewById(R.id.createButton);
        Button cancelButton = (Button) findViewById(R.id.cancelButton);
        cancelButton.setOnClickListener(
                new View.OnClickListener () {
                    public void onClick(View v){ finish();}
                }
        );
        
        //Cursor c = managedQuery(DataDefs.Fields.mkQueryUri(), null, null, null);
        placement = getIntent().getExtras().getInt("placement");
        Uri data = getIntent().getData();
        if(action.equals(ADD_LOC_ACTION)){
            
            setTitle("New Location Setup:");
            
            Spinner providerSpinner = (Spinner) findViewById(R.id.provider);
            ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this, R.array.providers, android.R.layout.simple_spinner_item);
            adapter.setDropDownViewResource(android.R.layout.simple_spinner_item);
            providerSpinner.setAdapter(adapter);

            createButton.setOnClickListener( new View.OnClickListener () {
                public void onClick(View v){onCreateLocClick(v);}
            });

        } else if (action.equals(ADD_ROOM_ACTION)){

            setTitle("New Room Setup:");
            
            loc = data.getPathSegments().get(1);
            createButton.setOnClickListener( new View.OnClickListener () {
                public void onClick(View v){onCreateRoomClick(v);}
            });

        } else if (action.equals(ADD_DEVICE_ACTION)){
            
            setTitle("New Device Setup:");
            
            loc = data.getPathSegments().get(1);
            room = data.getPathSegments().get(3);

            if(bonusDevice != null){
                device = bonusDevice;
                attempts = 1;
                devTestStart();
                return;
            }
            
            Spinner devTypeSpinner = (Spinner) findViewById(R.id.devicetype);
            ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this, R.array.devicetypes, android.R.layout.simple_spinner_item);
            adapter.setDropDownViewResource(android.R.layout.simple_spinner_item);
            devTypeSpinner.setAdapter(adapter);
            EditText brandname = (EditText) findViewById(R.id.brandname);
            if(icicle != null){
                devTypeSpinner.setSelection(icicle.getInt("device"));
                brandname.setText(icicle.getString("brand"));
            }
            
            createButton.setOnClickListener( new View.OnClickListener () {
                public void onClick(View v){
                    onDevNextClick();
                }
            });
            
        }
    }
    /** Method called when "continue" is clicked on location creation */
    void onCreateLocClick(View v){
        EditText locEditText = (EditText)findViewById(R.id.locname); 
        loc = locEditText.getText().toString();
        if(loc.equals("") || loc.equals(MenuBorgTV.newLocation)){
            emptyTextToast.show();
            locEditText.requestFocus();
            return;
        }
        Uri mURIl = getContentResolver().insert(DataDefs.Fields.mkInsertUri(loc,placement), null);
        Uri mURIr = getContentResolver().insert(DataDefs.Fields.mkInsertUri(loc, "Main Room",0), null);
        setResult(RESULT_OK, loc);
        successToast = Toast.makeText(this, "Location '"+loc+"' successfully added", Toast.LENGTH_SHORT);
        successToast.show();
        finish();
    }
    
    
    void onCreateRoomClick(View v){
        EditText roomEditText = (EditText)findViewById(R.id.roomname);
        room = roomEditText.getText().toString();
        if(room.equals("") || room.equals(MenuLocation.newRoom)){
            emptyTextToast.show();
            roomEditText.requestFocus();
            return;
        }
        Uri mURI = getContentResolver().insert(DataDefs.Fields.mkInsertUri(loc,room, placement), null);
        if(bonusDevice == null)
            setResult(RESULT_OK, room);
        else{
            Bundle b = new Bundle();
            b.putString("bonusDevice", bonusDevice);
            setResult(BONUS_DEV, room, b);
        }
        successToast = Toast.makeText(this, "Room '"+room+"' successfully added", Toast.LENGTH_SHORT);
        successToast.show();
        finish();
    }
    
    void onDevNextClick(){
        attempts = 1;
        Spinner deviceSpinner = ((Spinner)findViewById(R.id.devicetype));
        device = deviceSpinner.getSelectedItem().toString();
        deviceIndex = deviceSpinner.getSelectedItemPosition();
        EditText brandEditText = ((EditText)findViewById(R.id.brandname));
        brand = brandEditText.getText().toString();
        if(brand.equals("") || brand.equals(MenuRoom.newDevice)){
            emptyTextToast.show();
            brandEditText.requestFocus();
            return;
        }
        Cursor dCursor = this.managedQuery(DataDefs.Fields.mkQueryUri(loc, room), null, null, null, null);
        if(dCursor.first()){
            do{
                if(dCursor.getString(0).equals(device)){
                    AlertDialog.show(this, "Duplicate Device!",
                            R.drawable.alert, "A "+device+" device already exists in this Room. Would you "+
                            "like to create a new Room for this device?",
                            "Yes", new DialogInterface.OnClickListener(){public void onClick(DialogInterface d,int i){
                                onDevCollisionAccept();
                            }},
                            "No", new DialogInterface.OnClickListener(){public void onClick(DialogInterface d,int i){
                                finish();
                            }},
                            false, null);
                    return;//Should I return here?
                }
            } while(dCursor.next());
        }
        devTestStart();
    }
    
    void devTestStart(){
        inDevTestStart = true;
        setContentView(R.layout.setupdevicetesting);
        //TextView title = (TextView) findViewById(R.id.title);
        //title.setText("Configuring new "+device);
        setTitle("Configuring new "+device+"...");
        TextView attemptsLabel = (TextView) findViewById(R.id.attemptsLabel);
        attemptsLabel.setText(""+attempts);
        TextView attemptsMaxLabel = (TextView) findViewById(R.id.attemptsMaxLabel);
        attemptsMaxLabel.setText(" of 5");
        Button testButton = (Button) findViewById(R.id.testButton);
        Button cancelButton = (Button) findViewById(R.id.cancelButton);
        irToast = Toast.makeText(this, "Sending Test Signal "+ attempts +" of 5...", Toast.LENGTH_SHORT);
        testButton.requestFocus();
        testButton.setOnClickListener( new View.OnClickListener () {
           public void onClick(View v){
               irToast.show();
               inDevTestStart = false;
               onTestClick();
           } 
        });
        cancelButton.setOnClickListener( new View.OnClickListener () {
            public void onClick(View v){
                exitVerificationDialog();
            }
        });
    }
    
    void onTestClick(){
        inDevTestSend = true;
        setContentView(R.layout.setupdevicetesting2);
        Button noButton = (Button) findViewById(R.id.noButton);
        Button yesButton = (Button) findViewById(R.id.yesButton);
        yesButton.requestFocus();
        successToast = Toast.makeText(this, "Device '"+device+"' successfully added", Toast.LENGTH_SHORT);
        yesButton.setOnClickListener( new View.OnClickListener () {
           public void onClick(View v){
               ++attempts;
               if (attempts > 5){
                   maxAttemptsToast.show();
                   finish();
               }
               inDevTestSend = false;
               devTestStart();} 
        });
        noButton.setOnClickListener( new View.OnClickListener(){
            public void onClick(View v){
                Uri mURI = getContentResolver().insert(DataDefs.Fields.mkInsertUri(loc, room, device, placement), null);
                setResult(RESULT_OK, device);
                successToast.show();
                finish();
            }
        });
    }
    
    void exitVerificationDialog(){
        AlertDialog.show(this, "Are you sure?",
                R.drawable.alert, "Are you sure you want to cancel?",
                "Yes", new DialogInterface.OnClickListener(){public void onClick(DialogInterface d,int i){
                    finish();}},
                "No", new DialogInterface.OnClickListener(){public void onClick(DialogInterface d,int i){
                    return;
                }},
                false, null);
    }
    
    void onDevCollisionAccept(){
        this.setResult(DEVICE_COLLISION, device);
        finish();
    }
    
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK){
            if (inDevTestStart){
                if (attempts > 1){
                    inDevTestStart = false;
                    --attempts;
                    onTestClick();
                } else {
                    Bundle b = new Bundle();
                    b.putInt("device", deviceIndex);
                    b.putString("brand", brand);
                    onCreate(b);
                }
                return true;
            } else if (inDevTestSend){
                inDevTestSend = false;
                devTestStart();
                return true;
            } else {
                return super.onKeyDown(keyCode, event);
            }
        }
        return super.onKeyDown(keyCode, event);
    }

}
