package edu.berkeley.cs160.happytoastmobile.BorgTV.TVGuide;

// Totally stolen from the notepad tutorial

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;

import java.io.FileNotFoundException;

/**
 * Simple notes database access helper class. Defines the basic CRUD operations
 * for the notepad example, and gives the ability to list all notes as well as
 * retrieve or modify a specific note.
 * 
 * This has been improved from the first version of this tutorial through the addition
 * of better error handling and also using returning a Cursor instead of using
 * a collection of inner classes (which is less scalable and not recommended).
 */

public class GuideDbAdapter {

    public static final String KEY_FAVORITE="favorite";
    public static final String KEY_REMINDER="reminder";
    public static final String KEY_RECURRINGREMINDER="recurringReminder";    
    public static final String KEY_TITLE="title";
    public static final String KEY_TIME="time";
    public static final String KEY_DURATION="duration";
    public static final String KEY_ROWID="_id";
    public static final String KEY_CHANNELNAME="channel";
    public static final String KEY_HUMANTIME="humanTime";
    
    /**
     * Database creation sql statement
     */
    private static final String DATABASE_CREATE =
        "create table shows (_id integer primary key autoincrement, "
            + "title text not null, "
            + "time integer not null, "
            + "humanTime string not null, "
            + "duration integer not null default 30, "
            + "favorite integer not null default 0, "
            + "reminder integer not null default 0, "
            + "recurringReminder integer not null default 0, "
            + "channel text not null "
            +");";

    private static final String DATABASE_NAME = "guide";
    private static final String DATABASE_TABLE = "shows";
    private static final int DATABASE_VERSION = 4;
    
    private SQLiteDatabase mDb;
    private final Context mCtx;

    /**
     * Constructor - takes the context to allow the database to be opened/created
     * @param ctx the Context within which to work
     */
    public GuideDbAdapter(Context ctx) {
        this.mCtx = ctx;
    }
    
    
    public void killDatabase(){
    	mDb.execSQL("DROP TABLE IF EXISTS " + DATABASE_TABLE);
    }
    /**
     * Open the notes database. If it cannot be opened, try to create a new instance of
     * the database. If it cannot be created, throw an exception to signal the failure
     * @return this (self reference, allowing this to be chained in an initialization call)
     * @throws SQLException if the database could be neither opened or created
     */
    public GuideDbAdapter open() throws SQLException {
        try {
            mDb = mCtx.openDatabase(DATABASE_NAME, null);
        } catch (FileNotFoundException e) {
            try {
                mDb =
                    mCtx.createDatabase(DATABASE_NAME, DATABASE_VERSION, 0,
                        null);
                mDb.execSQL(DATABASE_CREATE);
            } catch (FileNotFoundException e1) {
                throw new SQLException("Could not create database");
            }
        }
        return this;
    }

    public void close() {
        mDb.close();
    }
    
    public boolean containsShow(Show show){
    	Cursor result = mDb.query(true, DATABASE_TABLE, new String[] {
                KEY_ROWID, KEY_FAVORITE, KEY_REMINDER, KEY_RECURRINGREMINDER},
                KEY_TITLE + "=? AND " + 
                KEY_TIME +"=? AND " +
                KEY_CHANNELNAME + "=?",
                new String[] {
	    			show.getTitle(), 
	    			(new Integer(show.getStartTime())).toString(), 
	    			show.getChannelName()
	    			}, 
	    		null, null, null);
        if ((result.count() == 0) || !result.first()) {
        	result.close();
        	return false;
        }
        result.close();
        return true;
    }
    
    public Cursor getRemindersCursor(){
    	Cursor result = mDb.query(true, DATABASE_TABLE, new String[] {
                KEY_ROWID, KEY_TITLE, KEY_HUMANTIME, KEY_TIME, KEY_CHANNELNAME, KEY_FAVORITE, KEY_REMINDER, KEY_RECURRINGREMINDER},
                KEY_REMINDER + "=? OR " + 
                KEY_RECURRINGREMINDER +"=?",
                new String[] {
	    			"1", "1"
    			}, 
	    		null, null, KEY_TIME);
    	if ((result.count() == 0) || !result.first()) {
        	result.close();
        	return null;
        }
    	return result;
    }
    
    public Cursor getFavoritesCursor(){
    	Cursor result = mDb.query(true, DATABASE_TABLE, new String[] {
                KEY_ROWID, KEY_TITLE, KEY_HUMANTIME, KEY_TIME, KEY_CHANNELNAME, KEY_FAVORITE, KEY_REMINDER, KEY_RECURRINGREMINDER},
                KEY_FAVORITE + "=?",
                new String[] {
	    			"1"
    			},
	    		null, null, KEY_TIME);
    	if ((result.count() == 0) || !result.first()) {
        	result.close();
        	return null;
        }
    	return result;
    }
    
    public Cursor getAiringsByName(String name){
    	Cursor result = mDb.query(true, DATABASE_TABLE, new String[] {
    			KEY_ROWID, KEY_TITLE, KEY_HUMANTIME, KEY_TIME, KEY_CHANNELNAME, KEY_FAVORITE, KEY_REMINDER, KEY_RECURRINGREMINDER},
                KEY_TITLE  +
                		" LIKE ?",
                new String[] {
	    			"%" + name + "%"
	    			}, 
	    		null, null, KEY_TIME);
    	//mDb.
        if ((result.count() == 0) || !result.first()) {
        	result.close();
        	return null;
        }
        return result;
    }
    
    public Cursor getShowCursor(Show show){
    	Cursor result = mDb.query(true, DATABASE_TABLE, new String[] {
                KEY_ROWID, KEY_FAVORITE, KEY_REMINDER, KEY_RECURRINGREMINDER},
                KEY_TITLE + "=? AND " + 
                KEY_TIME +"=? AND " +
                KEY_CHANNELNAME + "=?",
                new String[] {
	    			show.getTitle(), 
	    			(new Integer(show.getStartTime())).toString(), 
	    			show.getChannelName()
	    			}, 
	    		null, null, null);
        if ((result.count() == 0) || !result.first()) {
        	result.close();
        	return null;
        }
        return result;
    }


    public void addShow(Show show, boolean fav, boolean reminder, boolean recurring) {
    	ContentValues initialValues = new ContentValues();
    	
    	
        initialValues.put(KEY_TITLE,  show.getTitle() );
        initialValues.put(KEY_TIME, show.getStartTime());
        
        int hour = ( show.getStartTime()/100) % 12;
    	hour = (hour == 0) ? 12 : hour;
    	String meridian = ( show.getStartTime()/100 < 12) ? new String("am") : new String("pm");
    	String minute =  show.getStartTime()%100 == 30 ? new String("30") : new String("00");   	
        initialValues.put(KEY_HUMANTIME, hour + ":" + minute + meridian + " ");
        initialValues.put(KEY_CHANNELNAME,  show.getChannelName());
        initialValues.put(KEY_DURATION, show.getDuration());
        initialValues.put(KEY_FAVORITE, fav ? 1 : 0);
        initialValues.put(KEY_REMINDER, reminder ? 1 : 0);
        initialValues.put(KEY_RECURRINGREMINDER, recurring ? 1 : 0);
        mDb.insert(DATABASE_TABLE, null, initialValues);
        
    }
    
    
    
    
    public boolean getRecurringReminder(Show show) {
    	Cursor c = getShowCursor(show);
    	if (c == null){
    		return false;
    	}
    	c.first();
    	boolean reminder = c.getInt(c.getColumnIndex(KEY_RECURRINGREMINDER)) == 1;
    	c.close();
    	return reminder;
    }
    
    public void setRecurringReminder(Show show, boolean reminder){
        ContentValues args = new ContentValues();
        args.put(KEY_RECURRINGREMINDER, reminder ? 1 : 0);
        Cursor entry = getShowCursor(show);
        entry.first();
        long rowId = entry.getLong(entry.getColumnIndex(KEY_ROWID));
        entry.close();
        mDb.update(DATABASE_TABLE, args, KEY_ROWID + "=" + rowId, null);
    }
    
    public boolean getReminder(Show show) {
    	Cursor c = getShowCursor(show);
    	if (c == null){
    		return false;
    	}
    	c.first();
    	boolean reminder = c.getInt(c.getColumnIndex(KEY_REMINDER)) == 1;
    	c.close();
    	return reminder;
    }
    
    public void setReminder(Show show,  boolean reminder){
        ContentValues args = new ContentValues();
        args.put(KEY_REMINDER, reminder ? 1 : 0);
        Cursor entry = getShowCursor(show);
        entry.first();
        long rowId = entry.getLong(entry.getColumnIndex(KEY_ROWID)); 
        entry.close();
        mDb.update(DATABASE_TABLE, args, KEY_ROWID + "=" + rowId, null);
    }
    
    public boolean getFavorite(Show show) {
    	Cursor c = getShowCursor(show);
    	if (c == null){
    		return false;
    	}
    	c.first();
    	boolean fav = c.getInt(c.getColumnIndex(KEY_FAVORITE)) == 1;
    	c.close();
    	return fav;
    }
    
    public void setFavorite(Show show, boolean fav){
        ContentValues args = new ContentValues();
        args.put(KEY_FAVORITE, fav ? 1 : 0);
        Cursor entry = getShowCursor(show);
        entry.first();
        long rowId = entry.getLong(entry.getColumnIndex(KEY_ROWID));
        entry.close();
        mDb.update(DATABASE_TABLE, args, KEY_ROWID + "=" + rowId, null);
    }
    
    public void killShow(Show show){
    	mDb.delete(DATABASE_TABLE, 
                KEY_TITLE + "=? AND " + 
                KEY_TIME +"=? AND " +
                KEY_CHANNELNAME + "=?",
                new String[] {
	    			show.getTitle(), 
	    			(new Integer(show.getStartTime())).toString(), 
	    			show.getChannelName()
	    			});
    }
    
}


