package edu.berkeley.cs160.happytoastmobile.BorgTV.TVGuide;

import java.util.Calendar;


import edu.berkeley.cs160.happytoastmobile.BorgTV.R;
import android.app.Activity;
import android.app.AlarmManager;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.view.Window;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.TextView;
import android.widget.Toast;

public class ShowInfo extends Activity {
	
	private String title;
	private String channelName;
	private int channelNumber;
	private int duration;
	private int startTime;
	private String description;
	private boolean favorite;
	private boolean reminder;
	private boolean recurringReminder;
	
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        //this.requestWindowFeature(Window.FEATURE_NO_TITLE);
        setContentView(R.layout.showinfo);
        
		Intent intent = getIntent();
        channelName = intent.getStringExtra("channelName");
        channelNumber = intent.getIntExtra("channelNumber", -1);
        title = intent.getStringExtra("title");
        startTime = intent.getIntExtra("startTime", -1);
        duration = intent.getIntExtra("duration", -1);
        description = intent.getStringExtra("description");
        
        
        
        setTitle(title);
        TextView tv = (TextView) findViewById(R.id.showDescription);
        tv.setText(description);
        
        tv = (TextView) findViewById(R.id.showTime);
        int hour = (startTime/100) % 12;
    	hour = (hour == 0) ? 12 : hour;
    	String meridian = (startTime/100 < 12) ? new String("am") : new String("pm");
    	String minute = startTime%100 == 30 ? new String("30") : new String("00");   	
        tv.setText(hour + ":" + minute + meridian + " ");
        
        tv = (TextView) findViewById(R.id.showDuration);
        tv.setText(duration + " min");
        
        tv = (TextView) findViewById(R.id.channelName);
        tv.setText(channelName);
        
        tv = (TextView) findViewById(R.id.channelNumber);
        tv.setText("#" + channelNumber);
        
        // set up a listener for dealing with the button clicks
        OnClickListener buttonListener = new OnClickListener(){
			public void onClick(View v) {
				if (v == findViewById(R.id.showInfoBack)){
					onBack(); // ok button calls onOK
				} else if (v == findViewById(R.id.showInfoReminder)){
					onReminder(); // cancel button calls onCancel
				} else if (v == findViewById(R.id.showInfoRecurringReminder)){
					onRecurringReminder();
				} else if (v == findViewById(R.id.showInfoFavorite)){
					onFavorite();
				}
			}
        };
        
        favorite = intent.getBooleanExtra("favorite", false);
        reminder = intent.getBooleanExtra("reminder", false);
        recurringReminder = intent.getBooleanExtra("recurringReminder", false);

        // attach the listener to the buttons
        ImageButton button = (ImageButton) findViewById(R.id.showInfoBack);
        button.setOnClickListener(buttonListener);
        button.requestFocus();
        button = (ImageButton) findViewById(R.id.showInfoReminder);
        button.setOnClickListener(buttonListener);
        if (reminder)
        	button.setImageResource(R.drawable.no1bell);
        button = (ImageButton) findViewById(R.id.showInfoRecurringReminder);
        button.setOnClickListener(buttonListener);
        if (recurringReminder)
        	button.setImageResource(R.drawable.no3bell);
        button = (ImageButton) findViewById(R.id.showInfoFavorite);
        button.setOnClickListener(buttonListener);
        if (favorite)
        	button.setImageResource(R.drawable.noheart);
    }
 
    public void onBack(){
    	this.finishUp(RESULT_CANCELED);
    }
    
    public void onReminder(){
    	String time = (String) ((TextView) findViewById(R.id.showTime)).getText();
    	
    	String message = (!reminder) ? "A reminder has been set for " + time + 
    					" on " + channelName + ", channel " + channelNumber + "."
    					: "The next reminder for " + title + " has been cancelled.";
    	
    	
    	
    	Toast wtfIsToast = Toast.makeText(this, message, Toast.LENGTH_SHORT);
    	wtfIsToast.show();
    	reminder = !reminder;
    	if (reminder){
    		AlarmManager mgr = (AlarmManager) getSystemService(Context.ALARM_SERVICE);
    		Intent intent = new Intent(ShowInfo.this, ReminderIntentReceiver.class);
    		intent.putExtra("channelName", channelName);
    		intent.putExtra("showTitle", title);
    		intent.putExtra("channelNumber", channelNumber);
    		
    		
    		// Five minute warning alarm
    		Calendar c = Calendar.getInstance();
    		c.setTimeInMillis(System.currentTimeMillis());
    		c.set(Calendar.HOUR_OF_DAY, startTime/100);
    		c.set(Calendar.MINUTE, startTime%100);
    		c.add(Calendar.MINUTE, -5);
    		if (c.getTimeInMillis() < System.currentTimeMillis()){
    			c.add(Calendar.DAY_OF_YEAR, 7);
    		}
    		
    		// 10 seconds from now alarm
    		c.setTimeInMillis(System.currentTimeMillis());
    		c.add(Calendar.SECOND, 10);
    		
    		mgr.set(AlarmManager.RTC, c.getTimeInMillis(), intent);

    		
    	}
    	finishUp(RESULT_OK);
    }
    
    public void onRecurringReminder(){
    	String time = (String) ((TextView) findViewById(R.id.showTime)).getText();
    	
    	String message = (!recurringReminder) ? "A weekly reminder has been set for " + time + 
    					" on " + channelName + ", channel " + channelNumber + "."
    					: "All future reminders for " + title + " have been cancelled.";
    	Toast wtfIsToast = Toast.makeText(this, message, Toast.LENGTH_SHORT);
    	wtfIsToast.show();
    	recurringReminder = !recurringReminder;
    	finishUp(RESULT_OK);
    }
    
    public void onFavorite(){
    	String message = (!favorite) ? title + " has been added to your favorites." 
    			: title + " has been removed from your favorites.";
    	Toast wtfIsToast = Toast.makeText(this, message, Toast.LENGTH_SHORT);
    	wtfIsToast.show();
    	favorite = !favorite;
    	finishUp(RESULT_OK);
    }
    public void finishUp(int resultCode){
    	Bundle extras = new Bundle();
    	extras.putBoolean("favorite", favorite);
    	extras.putBoolean("recurringReminder", recurringReminder);
    	extras.putBoolean("reminder", reminder);
    	setResult(resultCode, null, extras);
    	finish();
    }
    
}
 
